
!Color class methodsFor: '*Tweak-Preload' stamp: 'ar 12/12/2009 00:57'!
fromHtmlColorRef: aString
	(aString beginsWith: '#') ifTrue:[^self fromHtmlColorRef: aString allButFirst].
	aString size = 6 ifFalse:[^self error: 'Not a color ref'].
	^self 
		r: (aString first digitValue * 16) + (aString second digitValue) / 255.0
		g: (aString third digitValue * 16) + (aString fourth digitValue) / 255.0
		b: (aString fifth digitValue * 16) + (aString sixth digitValue) / 255.0

! !

!XMLElement methodsFor: '*Tweak-Preload' stamp: 'ar 12/2/2003 21:30'!
asLiteral: aString
	| ch num |
	aString = 'nil' ifTrue:[^nil].
	aString = 'true' ifTrue:[^true].
	aString = 'false' ifTrue:[^false].

	ch := (aString at: 1).

	"strings"
	ch = $' ifTrue:[^self convertLiteralString: aString].

	"symbols"
	ch = $# ifTrue:[^self convertLiteralSymbol: aString].

	"globals"
	(ch isLetter and:[ch isUppercase]) ifTrue:[^Smalltalk at: (aString asSymbol)].

	"numbers/points"
	(ch = $- or:[ch isDigit]) ifTrue:[
		num := self convertLiteralNumeric: aString.
		num ifNotNil:[^num].
	].
	^Object readFrom: aString! !

!XMLElement methodsFor: '*Tweak-Preload' stamp: 'ar 12/2/2003 21:31'!
convertLiteralNumeric: aString
	"Convert aString from a literal number/point - this needs to be ultra-fast"
	| stream num ch |
	stream := ReadStream on: aString.
	num := Number readFrom: stream.
	ch := stream peek ifNil:[^num].
	ch = $@ ifTrue:[
		stream next.
		num := num @ (Number readFrom: stream).
	].
	stream atEnd ifTrue:[^num].
	"bail for everything else"
	^nil
! !

!XMLElement methodsFor: '*Tweak-Preload' stamp: 'ar 12/2/2003 21:30'!
convertLiteralString: aString
	"Convert aString from a literal string - this needs to be ultra-fast"
	| out nextIndex lastIndex |
	(aString at: 1) = $' ifFalse:[self error: 'string quote expected'].
	"do a quick match for a single-quoted string"
	nextIndex := aString indexOf: $' startingAt: 2.
	nextIndex = aString size ifTrue:[^aString copyFrom: 2 to: aString size-1].
	"do it the slow way"
	out := WriteStream on: (String new: aString size - 2).
	lastIndex := 1.
	[nextIndex := aString indexOf: $' startingAt: lastIndex+1.
	nextIndex = 0 ifTrue:[self error: 'string quote expected'].
	nextIndex < aString size] whileTrue:[
		out nextPutAll: (aString copyFrom: lastIndex+1 to: nextIndex-1).
		(aString at: nextIndex + 1) = $' ifFalse:[self error: 'string quote expected'].
		out nextPut: $'.
		lastIndex := nextIndex+1.
	].
	out nextPutAll: (aString copyFrom: lastIndex+1 to: aString size-1).
	^out contents! !

!XMLElement methodsFor: '*Tweak-Preload' stamp: 'ar 12/2/2003 21:31'!
convertLiteralSymbol: aString
	"Convert aString from a literal symbol - this needs to be ultra-fast"
	(aString at: 1) = $# ifFalse:[self error: 'hash mark expected'].
	(aString at: 2) = $' ifFalse:[^Symbol intern: (aString copyFrom: 2 to: aString size)].
	^(self convertLiteralString: (aString copyFrom: 2 to: aString size)) asSymbol
! !

!XMLElement methodsFor: '*Tweak-Preload' stamp: 'ar 12/2/2003 21:30'!
literalAt: key
	| attr |
	attr := self attributeAt: key ifAbsent:[self errorNotFound: key].
	^self asLiteral: attr! !
